# -*- coding: utf-8 -*-

import urlparse
import uuid

import requests

import api_account.run
import api_account.views
import api_compute.run
import api_compute.views
import api_metadata.views
import httmock
from ocs.api.app import Application
from ocs.api.unittest import FlaskAPITestCase
from ocs.conf import Configuration
from ocs.object_store import cp_backend, world_backend
from ocs.object_store.cp_backend.unittest import ComputeDatabaseTestCase
from ocs.object_store.world_backend.unittest import sources as world_sources
from ocs.object_store.world_backend.unittest import WorldDatabaseTestCase
from scaleway.apis import AccountAPI, ComputeAPI


class APIMixin(object):

    def __init__(self, app):
        app.testing = True
        self.client = app.flask_app.test_client()

    def proxy_requests(self, api_url, client, url, request):
        """ Proxies `request` to `client` if `url` is a part of `api_url`.
        """
        request_url = urlparse.urlunsplit(url)

        # This request must not be proxified to this client
        if not request_url.startswith(api_url):
            return None

        if request.method == 'GET':
            response = self.client.get(
                request.path_url, headers=dict(request.headers)
            )

            return httmock.response(
                response.status_code,
                response.data,
                headers=dict(response.headers)
            )
        elif request.method in ('DELETE', 'PATCH', 'PUT'):
            response = getattr(self.client, request.method.lower())(
                request.path_url,
                data=request.body,
                headers=dict(request.headers)
            )

            return httmock.response(
                response.status_code,
                response.data,
                headers=dict(response.headers)
            )

        raise NotImplementedError(
            'HTTP verb %s not implemented in proxy' % request.method
        )

    def create_token(self, secret, service, name, resource):
        """ Creates a token and associates permissions to it.
        """
        user = world_sources.UserFactory()
        service = world_sources.ServiceFactory(name=service)

        permission = world_sources.PermissionFactory(
            service=service, name=name, resource=resource
        )

        world_sources.UsersPermissionsFactory(user=user, permission=permission)

        token = world_sources.TokenFactory(
            secret=secret,
            user=user,
            use_user_roles=True
        )

        world_backend.ConfiguredEngine.scoped_session().commit()

        return token


class FakeComputeAPI(APIMixin):

    # The api-metadata requires a token that can read all servers
    PRIVILEGED_TOKEN = str(uuid.uuid4())

    def __init__(self):
        # Create the fake application
        app = Application(api_compute.run.APPLICATION_INFO)
        app.register_views(api_compute.views)
        app.setup_dbsession(world_backend.ConfiguredEngine.scoped_session)
        app.setup_dbsession(cp_backend.ConfiguredEngine.scoped_session)
        self.load_data()
        super(FakeComputeAPI, self).__init__(app)

        # Configure ocs.conf
        Configuration.load_from_dict({
            'api-compute': {
                'api-account': {
                    'token': FakeAccountAPI.PRIVILEGED_TOKEN,
                },
                'api-task': {
                    'token': 'token-api-task-TBD',
                    'url': 'http://url-api-task.tbd/',
                },
            }
        }, api_compute.views.CONFIGURATION_NAME)

    def load_data(self):
        """ Instanciates models needed to make the API working.
        """
        # Create a token that can do anything on the compute service. In
        # production, the token used should only be able to READ data.
        self.create_token(self.PRIVILEGED_TOKEN, 'compute', '*', '*')

    def proxy_requests(self, url, request):
        return super(FakeComputeAPI, self).proxy_requests(
            ComputeAPI().get_api_url(), self.client,
            url, request
        )


class FakeAccountAPI(APIMixin):

    # The api-metadata requires a token that can read all organizations
    PRIVILEGED_TOKEN = str(uuid.uuid4())

    def __init__(self):
        # Create the fake application
        app = Application(api_account.run.APPLICATION_INFO)
        app.register_views(api_account.views)
        app.setup_dbsession(world_backend.ConfiguredEngine.scoped_session)
        self.load_data()
        super(FakeAccountAPI, self).__init__(app)

    def load_data(self):
        self.create_token(self.PRIVILEGED_TOKEN, 'account', '*', '*')

    def proxy_requests(self, url, request):
        return super(FakeAccountAPI, self).proxy_requests(
            AccountAPI().get_api_url(), self.client,
            url, request
        )


class APITestCase(FlaskAPITestCase,
                  WorldDatabaseTestCase, ComputeDatabaseTestCase):

    CLIENT_IP_ADDRESS = '10.0.0.100'

    def setUp(self):
        super(APITestCase, self).setUp()

        self.fake_compute_api = FakeComputeAPI()
        self.fake_account_api = FakeAccountAPI()

        self.app.register_views(api_metadata.views)

        # Give tokens to api-metadata to query api-compute and api-account
        Configuration.load_from_dict({
            'api-metadata': {
                'api-compute': {
                    'token': FakeComputeAPI.PRIVILEGED_TOKEN,
                },
                'api-account': {
                    'token': FakeAccountAPI.PRIVILEGED_TOKEN,
                },
            }
        }, api_metadata.views.CONFIGURATION_NAME)

        Configuration.load_from_dict({}, 'ocs')

    def get_environ(self, environ):
        environ = super(APITestCase, self).get_environ(environ)
        environ['REMOTE_ADDR'] = self.CLIENT_IP_ADDRESS
        return environ

    def default_requests_send(self, mock, request):
        """ Escapes from the HTTMock context to really make the HTTP request.
        """
        # HTTMock monkeypatches requests.Session.send

        # keep the patched value for later
        patched_send = requests.Session.send
        # unpatch requests
        requests.Session.send = mock._real_session_send

        # make the real HTTP request
        try:
            session = requests.Session()
            res = session.send(request)
        except:
            raise
        finally:
            # restore the patch
            requests.Session.send = patched_send

        return res

    def run(self, *args, **kwargs):
        """ Forwards HTTP requests to FakeComputeAPI, or to FakeAccountAPI, or
        fallback to a real request.
        """
        with httmock.HTTMock(
            lambda url, request:
                self.fake_compute_api.proxy_requests(url, request) or
                self.fake_account_api.proxy_requests(url, request) or
                self.default_requests_send(mock, request)
        ) as mock:

            return super(APITestCase, self).run(*args, **kwargs)
