""" Utilities to modify the default views response format.

This code is used ONLY for compat. It has been imported from
ocs-api@502fafdd9a1e65b1a1b11962 and it SHOULD NOT be modified.

See ConfView.conf for more details.
"""

import functools
import json
import pipes

from flask import request


def shell(response):
    """ Flattens the response JSON-dict data as a list of key-value paris.

    Updates the Content-Type header to "text/plain".

    For each key/value of the dictionary:
    - "Simple" elements (strings, bools, integers) are represented as a string
      "KEY=value". pipes.quote is used to handle escaping.
    - Lists are represented as a string "KEY=n" where n is the number of
      elements in the list, then with one line per element "KEY_n=value" where
      n is the index of the element.
    - Dictionaries are represented as a string "KEY=a b c" where a, b, c are
      the keys of the dictionary, then with one line per element "KEY_X=value"
      where X is the dictionary key.

    Example:

    >>> shell({
    ...     'hello': 'world',
    ...     'complex': 'value with spaces',
    ...     'list': ['one', 'two'],
    ...     'dict': {
    ...         'first': 'value',
    ...         'second': 'value with spaces',
    ...         'third': ['x', 'xx', 'xxx']
    ...     }
    ... })
    HELLO=world
    COMPLEX='value with spaces'
    LIST=2
    LIST_0=one
    LIST_1=two
    DICT=FIRST SECOND THIRD
    DICT_FIRST=value
    DICT_SECOND='value with spaces'
    DICT_THIRD=3
    DICT_THIRD_0=x
    DICT_THIRD_1=xx
    DICT_THIRD_2=xxx
    """
    def _transform_line(key, value):
        """ Transforms a single line (simple type, list or dict).
        """
        # All keys are in uppercase
        def make_key(key):
            """ Make key """
            return key.upper()

        if isinstance(value, dict):
            # KEY=a b c
            out = '%s=%s\n' % (
                make_key(key), ' '.join(make_key(key) for key in value.keys())
            )
            for elem_key, elem_value in value.items():
                out += _transform_line(
                    make_key('%s_%s' % (key, elem_key)),
                    elem_value
                )
            return out

        elif isinstance(value, list):
            # KEY=n
            out = '%s=%s\n' % (make_key(key), len(value))
            for idx, elem in enumerate(value):
                out += _transform_line(make_key('%s_%s' % (key, idx)), elem)
            return out

        # pipes.quote doesn't accept integers or bools, force unicode
        value = unicode(value)
        return '%s=%s\n' % (make_key(key), pipes.quote(value))

    response.headers['Content-Type'] = 'text/plain'
    response.data = ''.join(
        _transform_line(key, value)
        for key, value in json.loads(response.data).items()
    )
    return response


def select_export(default=None):
    """ Captures the querystring argument "format" to change the response
    output.

    Possible formats:
    - sh (ocs.api.exports.shell)
    - anything else doesn't change the default behaviour.

    :param default: the format used if the querystring argument "format" is not
    provided.
    """
    def decorator(view):

        @functools.wraps(view)
        def wrapped(*args, **kwargs):
            # call view
            response = view(*args, **kwargs)

            # get the export function
            export_type = request.args.get('format')
            # not in querystring, or value is empty
            if not export_type:
                export_type = default

            exports = {
                'sh': shell,
            }
            export = exports.get(export_type)

            if export:
                return export(response)
            return response

        return wrapped
    return decorator
