from flask import current_app
from jinja2 import Environment, PackageLoader

from ocs.api.exceptions import NotFound

from . import MetadataAPIBaseView, client_ip


class IPXEView(MetadataAPIBaseView):
    """ Generates the iPXE configuration for a client.

    At the moment, bootscripts aren't tied to an architecture. A C1 (arm)
    server could generate its iPXE configuration - which is useless ; and a C2
    server could use a C1 bootscript - but it won't boot.

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    SECURITY NOTE: we must be extra cautious and not let the kernel, initrd and
    bootcmdargs be user defined without validation.
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    """
    route_base = '/'

    # TODO Add caching on bootscript uuids
    def _get_rescue_bootscript(self, arch):
        # Api only allow filter on title + arch
        response = self.privileged_compute_api.query().bootscripts.get(
            arch=arch,
            title='rescue',
            public='true')
        bootscripts = response.get('bootscripts', [])
        if len(bootscripts) == 0:
            raise NotFound('No rescue bootscript for %s' % arch)
        if len(bootscripts) > 1:
            raise NotFound('More than one rescue bootscript for %s' % arch)
        return bootscripts[0]

    def ipxe(self):
        ip_addr = client_ip()
        server = self._get_server_by_ip(ip_addr)

        server_boot_type = server.get('boot_type')
        if server_boot_type == 'rescue':
            bootscript = self._get_rescue_bootscript(server.get('arch'))
        else:
            bootscript = server.get('bootscript')

        # image's bootscript
        if bootscript is None:
            image = server.get('image')
            if image:
                bootscript = image.get('default_bootscript')

        # should never happen, since a server is required to have a bootscript.
        if bootscript is None:
            return current_app.response_class(
                'Bootscript not found', status=404, mimetype='text/plain'
            )

        module_name = '.'.join(__name__.split('.')[:-1])
        jinja_env = Environment(loader=PackageLoader(module_name, 'templates'))
        template = jinja_env.get_template('boot.ipxe')

        # Replace LINUX_COMMON by host specific value
        bootcmdargs = bootscript.get('bootcmdargs', '')

        commercial_type = server['commercial_type']
        if (
            commercial_type.startswith("VC") or
            commercial_type.startswith("ST") or
            commercial_type.startswith("X64") or
            commercial_type.startswith("ARM") or
            commercial_type.startswith("RENDER")
        ):
            bootcmdargs = bootcmdargs.replace(
                "LINUX_COMMON",
                "initrd=initrd showopts console=ttyS0,115200 nousb vga=0 "
                "root=/dev/vda"
            )
            # Fix the serial for arm64
            if commercial_type.startswith("ARM"):
                bootcmdargs = bootcmdargs.replace("ttyS0", "ttyAMA0")

        elif commercial_type.startswith("C2"):
            bootcmdargs = bootcmdargs.replace(
                "LINUX_COMMON",
                "showopts console=ttyS1,9600n8 nousb vga=0 "
                "root=/dev/nbd0"
            )

        result = template.render(
            kernel=bootscript.get('kernel'),
            initrd=bootscript.get('initrd'),
            bootcmdargs=bootcmdargs
        )
        return current_app.response_class(result, mimetype='text/plain')
